@@ -62,13 +62,16 @@ struct STATVFS statbuf;
 memset(&statbuf, 0, sizeof(statbuf));
 
 /* The field names are macros, because not all OS have F_FAVAIL and it seems
-tidier to have a macro for F_BAVAIL as well. */
+tidier to have a macro for F_BAVAIL as well. Some kinds of file server do not
+have inodes, and they return -1 for the number available, so we do the check
+only when this field is non-negative. */
 
 if (check_spool_space > 0 || msg_size > 0 || check_spool_inodes > 0)
   {
   if (STATVFS(spool_directory, &statbuf) != 0 ||
       statbuf.F_BAVAIL < (check_spool_space + msg_size) / statbuf.F_FRSIZE ||
-      statbuf.F_FAVAIL < check_spool_inodes) rc = FALSE;
+      (statbuf.F_FAVAIL >= 0 && statbuf.F_FAVAIL < check_spool_inodes))
+    rc = FALSE;
   DEBUG(5) debug_printf("spool directory %s space = %d blocks; inodes = %d; "
     "check_space = %d (%d blocks); inodes = %d; msg_size = %d (%d blocks)\n",
     spool_directory, statbuf.F_BAVAIL, statbuf.F_FAVAIL, check_spool_space,
@@ -2061,7 +2064,8 @@ if (!ferror(data_file) && !(accept_feof)(fin) && message_ended != END_DOT)
     goto TIDYUP;                       /* Skip to end of function */
     }
 
-  /* Handle message that is too big */
+  /* Handle message that is too big. Don't use host_or_ident() in the log
+  message; we want to see the ident value even for non-remote messages. */
 
   if (message_ended == END_SIZE)
     {
@@ -2070,14 +2074,13 @@ if (!ferror(data_file) && !(accept_feof)(fin) && message_ended != END_DOT)
       {
       accept_swallow_smtp(fin);
       smtp_reply = "552 Message too large";
-      log_write(2, LOG_MAIN|LOG_REJECT, "rejected%s%s%s%s%s <%s>: "
+      log_write(2, LOG_MAIN|LOG_REJECT, "rejected from <%s>%s%s%s%s: "
         "message too large: %d recipient%s size=%d max=%d",
-        (sender_fullhost == NULL)? "" : " from ",
-        (sender_fullhost == NULL)? "" : sender_fullhost,
-        (sender_ident == NULL)? "" : " (",
-        (sender_ident == NULL)? "" : sender_ident,
-        (sender_ident == NULL)? "" : ")",
         sender_address,
+        (sender_fullhost == NULL)? "" : " H=",
+        (sender_fullhost == NULL)? "" : sender_fullhost,
+        (sender_ident == NULL)? "" : " U=",
+        (sender_ident == NULL)? "" : sender_ident,
         recipients_count,
         (recipients_count == 1)? "" : "s",
         message_size,
@@ -2351,29 +2354,29 @@ else
   if (log_subject && subject_header != NULL)
     {
     int i;
-    char *p, *ss;
+    char *p;
     char *s = subject_header->text + 7;
 
     while (*s != ':') s++;
     while (isspace((uschar)*(++s)));
-    ss = string_printing(s);
 
     strcpy(big_buffer, " T=\"");
     p = big_buffer + (int)strlen(big_buffer);
 
-    /* Backslash-quote any double quotes or backslashes. If the string is
-    not now empty, it will end in "\n", since all headers end with \n and
-    string_printing() will have turned that into two characters. But if it
-    was an empty subject the terminating \n was removed by isspace() above. */
+    /* Backslash-quote any double quotes or backslashes so as to make a
+    a C-like string, and leave off the terminating newline. However, if
+    the subject was empty, the isspace() above will have skipped over
+    the newline, so we must first test for this case. Then turn any
+    non-printers into escape sequences. */
 
-    if (*ss != 0) for (i = 0; i < 100 && ss[i+2] != 0; i++)
+    if (*s != 0) for (i = 0; i < 100 && s[i+1] != 0; i++)
       {
-      if (ss[i] == '\"' || ss[i] == '\\') *p++ = '\\';
-      *p++ = ss[i];
+      if (s[i] == '\"' || s[i] == '\\') *p++ = '\\';
+      *p++ = s[i];
       }
     *p++ = '\"';
     *p = 0;
-    subject = big_buffer;
+    subject = string_printing(big_buffer);
     }
   else subject = "";
 
@@ -2464,7 +2467,7 @@ if (smtp_input)
         fprintf(stdout,
           "\n**** SMTP testing: that is not a real message id!\n\n");
       }
-    else smtp_printf("%s\r\n", smtp_reply);
+    else smtp_printf("%.1024s\r\n", smtp_reply);
     fflush(smtp_out);
     }
 
