@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 #include "../exim.h"
@@ -56,6 +56,7 @@ pam_converse (int num_msg, PAM_CONVERSE_ARG2_TYPE **msg,
   struct pam_response **resp, void *appdata_ptr)
 {
 int i;
+int sep = 0;
 struct pam_response *reply = malloc(sizeof(struct pam_response) * num_msg);
 
 if (reply == NULL) return PAM_CONV_ERR;
@@ -67,7 +68,7 @@ for (i = 0; i < num_msg; i++)
     {
     case PAM_PROMPT_ECHO_ON:
     case PAM_PROMPT_ECHO_OFF:
-    arg = string_nextinlist(&pam_args, ':', big_buffer, big_buffer_size);
+    arg = string_nextinlist(&pam_args, &sep, big_buffer, big_buffer_size);
     if (arg == NULL) arg = "";
     reply[i].resp = string_copy_malloc(arg); /* PAM frees resp */
     reply[i].resp_retcode = PAM_SUCCESS;
@@ -114,6 +115,7 @@ auth_call_pam(char *s, char **errptr)
 pam_handle_t *pamh = NULL;
 struct pam_conv pamc;
 int pam_error;
+int sep = 0;
 char *user;
 
 /* Set up the input data structure: the address of the conversation function,
@@ -130,7 +132,7 @@ pam_conv_had_error = FALSE;
 
 /* The first string in the list is the user. */
 
-user = string_nextinlist(&pam_args, ':', big_buffer, big_buffer_size);
+user = string_nextinlist(&pam_args, &sep, big_buffer, big_buffer_size);
 if (user == NULL) user = "";
 
 /* Start off PAM interaction */
