@@ -768,9 +768,12 @@ if (daemon_listen)
         if (ipv6_address)
           rc = inet_pton(AF_INET6, ipa->address, &sin.v6.sin6_addr);
         else
+        /* We have an IPv6 socket, and an IPv4 address. We have to turn the
+        address into an IPv6 mapped form. */
           {
-          rc = inet_pton(AF_INET, ipa->address, &sin.v4.sin_addr);
-          sin.v4.sin_family = AF_INET;
+          memmove(ipa->address+7, ipa->address, (int)strlen(ipa->address) + 1);
+          memcpy(ipa->address, "::ffff:", 7);
+          rc = inet_pton(AF_INET6, ipa->address, &sin.v6.sin6_addr);
           }
         if (rc != 1)
           log_write(0, LOG_PANIC_DIE, "unable to parse \"%s\"", ipa->address);
