@@ -416,9 +416,8 @@ open_db dbblock[8];
 int max_db = sizeof(dbblock)/sizeof(open_db);
 int current = -1;
 int showtime = 0;
-int i, fd;
-char *s;
-dbdata_wait *dbwait;
+int i;
+dbdata_wait *dbwait = NULL;
 char buffer[256];
 char structbuffer[1024];
 
@@ -476,14 +475,14 @@ while (fgets(buffer, 256, stdin) != NULL)
   clock_t start = 1;
   clock_t stop = 0;
   char *cmd = buffer;
-  while (len > 0 && isspace(buffer[len-1])) len--;
+  while (len > 0 && isspace((uschar)buffer[len-1])) len--;
   buffer[len] = 0;
 
-  if (isdigit(*cmd))
+  if (isdigit((uschar)*cmd))
     {
     count = atoi(cmd);
-    while (isdigit(*cmd)) cmd++;
-    while (isspace(*cmd)) cmd++;
+    while (isdigit((uschar)*cmd)) cmd++;
+    while (isspace((uschar)*cmd)) cmd++;
     }
 
   if (strncmp(cmd, "open", 4) == 0)
@@ -491,7 +490,7 @@ while (fgets(buffer, 256, stdin) != NULL)
     int i;
     open_db *odb;
     char *s = cmd + 4;
-    while (isspace(*s)) s++;
+    while (isspace((uschar)*s)) s++;
 
     for (i = 0; i < max_db; i++)
       if (dbblock[i].dbptr == NULL) break;
@@ -519,7 +518,7 @@ while (fgets(buffer, 256, stdin) != NULL)
 
   else if (strncmp(cmd, "write", 5) == 0)
     {
-    int rc;
+    int rc = 0;
     char *key = cmd + 5;
     char *data;
 
@@ -529,11 +528,11 @@ while (fgets(buffer, 256, stdin) != NULL)
       continue;
       }
 
-    while (isspace(*key)) key++;
+    while (isspace((uschar)*key)) key++;
     data = key;
-    while (*data != 0 && !isspace(*data)) data++;
+    while (*data != 0 && !isspace((uschar)*data)) data++;
     *data++ = 0;
-    while (isspace(*data)) data++;
+    while (isspace((uschar)*data)) data++;
 
     dbwait = (dbdata_wait *)(&structbuffer);
     strcpy(dbwait->text, data);
@@ -554,7 +553,7 @@ while (fgets(buffer, 256, stdin) != NULL)
       printf("No current database\n");
       continue;
       }
-    while (isspace(*key)) key++;
+    while (isspace((uschar)*key)) key++;
     start = clock();
     while (count-- > 0)
       dbwait = (dbdata_wait *)dbfn_read_with_length(dbblock+ current, key, NULL);
@@ -570,7 +569,7 @@ while (fgets(buffer, 256, stdin) != NULL)
       printf("No current database\n");
       continue;
       }
-    while (isspace(*key)) key++;
+    while (isspace((uschar)*key)) key++;
     dbfn_delete(dbblock + current, key);
     }
 
@@ -601,7 +600,7 @@ while (fgets(buffer, 256, stdin) != NULL)
   else if (strncmp(cmd, "close", 5) == 0)
     {
     char *s = cmd + 5;
-    while (isspace(*s)) s++;
+    while (isspace((uschar)*s)) s++;
     i = atoi(s);
     if (i >= max_db || dbblock[i].dbptr == NULL) printf("Not open\n"); else
       {
@@ -616,7 +615,7 @@ while (fgets(buffer, 256, stdin) != NULL)
   else if (strncmp(cmd, "file", 4) == 0)
     {
     char *s = cmd + 4;
-    while (isspace(*s)) s++;
+    while (isspace((uschar)*s)) s++;
     i = atoi(s);
     if (i >= max_db || dbblock[i].dbptr == NULL) printf("Not open\n");
       else current = i;
@@ -639,7 +638,8 @@ while (fgets(buffer, 256, stdin) != NULL)
   else printf("Eh?\n");
 
   if (showtime && stop >= start)
-    printf("start=%d stop=%d difference=%d\n", start, stop, stop - start);
+    printf("start=%d stop=%d difference=%d\n", (int)start, (int)stop,
+     ( int)(stop - start));
 
   printf("> ");
   }
