@@ -171,7 +171,7 @@ if (ob->new_address == NULL)
 
 
 /* A new_address string was specified. First, expand it. Expansion failure is
-either a panic or a director fail, according to an option. A forced fail, or
+either a panic or a director decline, according to an option. A forced fail, or
 search defer in the expansion is always soft. */
 
 raw_new_address = expand_string(ob->new_address);
@@ -192,9 +192,9 @@ if (raw_new_address == NULL)
     {
     DEBUG(4) debug_printf("expansion of \"%s\" failed: %s\n",
       ob->new_address, expand_string_message);
-    DEBUG(2) debug_printf("%s director failed for %s\n", dblock->name,
+    DEBUG(2) debug_printf("%s director declined for %s\n", dblock->name,
       addr->local_part);
-    return FAIL;
+    return DECLINE;
     }
   }
 
@@ -238,6 +238,7 @@ if (transport != NULL)
 
   /* Put in new values, and connect to parent. */
 
+  generated->onetime_parent = NULL;            /* Mustn't have this */
   generated->errors_address = errors_to;
   generated->extra_headers = extra_headers;
   generated->remove_headers = remove_headers;
