@@ -53,6 +53,16 @@ are needed by any of the other headers, including system headers. */
 #define UCHAR_MAX 255
 #endif
 
+/* Some systems have PATH_MAX and some have MAX_PATH_LEN. */
+
+#ifndef PATH_MAX
+#ifdef MAX_PATH_LEN
+#define PATH_MAX MAX_PATH_LEN
+#else
+#define PATH_MAX 1024
+#endif
+#endif
+
 #include <sys/types.h>
 #include <dirent.h>
 #include <netdb.h>
