@@ -77,6 +77,7 @@ static var_entry var_table[] = {
   { "address_pipe",        vtype_stringptr,   &address_pipe },
   { "authenticated_id",    vtype_stringptr,   &authenticated_id },
   { "authenticated_sender",vtype_stringptr,   &authenticated_sender },
+  { "body_linecount",      vtype_int,         &body_linecount },
   { "caller_gid",          vtype_int,         &real_gid },
   { "caller_uid",          vtype_int,         &real_uid },
   { "compile_date",        vtype_stringptr,   &version_date },
@@ -1472,7 +1473,6 @@ while (*s != 0)
       expand_string_message = "File lookups are not permitted";
       goto EXPAND_FAILED;
       }
-    lookup_value = NULL;
 
     /* Get the key we are to look up for single-key+file style lookups.
     Otherwise set the key NULL pro-tem. */
@@ -1590,20 +1590,22 @@ while (*s != 0)
       filename = NULL;
       }
 
-    /* If skipping, don't do the next bit - it has the result of leaving
-    lookup_value == NULL, as if the entry was not found. Note that there is no
-    search_close() function. Files are left open in case of re-use. At suitable
-    places in higher logic, search_tidyup() is called to tidy all open files.
-    This can save opening the same file several times. However, files may also
-    get closed when others are opened, if too many are open at once. The rule
-    is that a handle should not be used after a second search_open().
+    /* If skipping, don't do the next bit - just lookup_value == NULL, as if
+    the entry was not found. Note that there is no search_close() function.
+    Files are left open in case of re-use. At suitable places in higher logic,
+    search_tidyup() is called to tidy all open files. This can save opening
+    the same file several times. However, files may also get closed when others
+    are opened, if too many are open at once. The rule is that a handle should
+    not be used after a second search_open().
 
     Request that a partial search sets up $1 and maybe $2 by passing
     expand_setup containing zero. If its value changes, reset expand_nmax,
     since new variables will have been set. Note that at the end of this
     "lookup" section, the old numeric variables are restored. */
 
-    if (!skipping)
+    if (skipping)
+      lookup_value = NULL;
+    else
       {
       void *handle = search_open(filename, stype, 0, NULL, NULL,
         &expand_string_message);
@@ -1646,6 +1648,11 @@ while (*s != 0)
     if (looked_up)
       yield = string_cat(yield, &size, &ptr, sub1, (int)strlen(sub1));
 
+    /* Restore $value to what it was on entry so that it has this value
+    during the fail string. */
+
+    lookup_value = save_lookup_value;
+
     /* There now follows either another substring, or "fail", or nothing.
     This time, forced failures are noticed only if the original lookup failed.
     We must set skipping in the nested call if this lookup succeeded, or if
@@ -1695,14 +1702,10 @@ while (*s != 0)
     if (*s++ != '}') goto EXPAND_FAILED_CURLY;
 
     /* Restore external setting of expansion variables for carrying on
-    at this level. */
+    at this level, and continue. */
 
     restore_expand_strings(save_expand_nmax, save_expand_nstring,
       save_expand_nlength);
-
-    /* Restore lookup_value and continue */
-
-    lookup_value = save_lookup_value;
     continue;
     }
 
@@ -2389,8 +2392,18 @@ while (*s != 0)
 
   /* Else there's something wrong */
 
-  expand_string_message = string_sprintf("\"${%s%.3s\" unrecognized", name,
-    (*(--s) == 0)? "" : s);
+  expand_string_message =
+    string_sprintf("\"${%s\" is not a known operator (or a } is missing "
+    "in a variable reference)", name);
+  goto EXPAND_FAILED;
+  }
+
+/* If we hit the end of the string when ket_ends is set, there is a missing
+terminating brace. */
+
+if (ket_ends && *s == 0)
+  {
+  expand_string_message = "missing } at end of string";
   goto EXPAND_FAILED;
   }
 
