@@ -568,12 +568,14 @@ for (;;)
       /* Handle "does|is [not]", preserving the pointer after "is" in
       case it isn't that, but the form "is <string>". */
 
-      if (strcmp(buffer, "does") == 0 || strcmp(buffer, "is") == 0)
+      if (strcmpic(buffer, "does") == 0 || strcmpic(buffer, "is") == 0)
         {
-        if (buffer[0] == 'i') isptr = ptr;
+        if (buffer[0] == 'i') { c->type = cond_is; isptr = ptr; }
+        if (buffer[0] == 'I') { c->type = cond_IS; isptr = ptr; }
+
         ptr = nextword(ptr, buffer, sizeof(buffer), TRUE);
         if (*error_pointer != NULL) break;
-        if (strcmp(buffer, "not") == 0)
+        if (strcmpic(buffer, "not") == 0)
           {
           c->testfor = !c->testfor;
           if (isptr != NULL) isptr = ptr;
@@ -598,7 +600,6 @@ for (;;)
         {
         if (isptr != NULL)
           {
-          c->type = cond_is;
           ptr = isptr;
           }
         else
@@ -1853,12 +1854,13 @@ while (commands != NULL)
 
       /* Create the new address and add it to the chain, setting the
       ignore_error flag if necessary, and the errors address, which can be
-      set in a system filter. Ensure that the address is placed in the
-      MAIN storage pool. */
+      set in a system filter and to the local address in user filters. Ensure
+      that the addresses are placed in the MAIN storage pool. */
 
       store_pool = POOL_MAIN;
       addr = deliver_make_addr(expargs[0], TRUE);  /* TRUE => copy s */
-      addr->errors_address = s;                    /* Default is NULL */
+      addr->errors_address = (s == NULL)?
+        s : string_copy(s);                        /* Default is NULL */
       addr->ignore_error = commands->noerror;
       addr->next = *generated;
       *generated = addr;
@@ -2120,7 +2122,8 @@ while (commands != NULL)
         }
       else DEBUG(10) debug_printf("Filter: freeze \"%s\"\n", filter_fmsg);
       }
-    return FALSE;
+    /* Do NOT set "delivered" for freeze or fail. */
+    return FALSE;        /* (This is needed for -bF testing) */
 
     case finish_command:
     if (commands->seen) *delivered = TRUE;
@@ -2197,8 +2200,8 @@ while (commands != NULL)
           }
         else if (i < mailarg_index_text && c == '\n' && !isspace((uschar)p[1]))
           {
-          *error_pointer = string_sprintf("\\n not followed by space in \"%s\" "
-            "in %s command", string_printing(s),
+          *error_pointer = string_sprintf("\\n not followed by space in "
+            "\"%.1024s\" in %s command", string_printing(s),
             command_list[commands->command]);
           return FALSE;
           }
@@ -2333,7 +2336,7 @@ Arguments:
   delivered   points to the "delivered" state of the filtering
   action      if non-null, allow special actions and return here
   error       where to pass back an error text
-  log_access  0 => OK to obey the "log" command
+  log_access  0 => OK to obey the "logwrite" command
               1 => just skip it (verifying or testing)
               2 => locked out by configuration
               3 => not running as user
@@ -2390,7 +2393,12 @@ if (read_command_list(&ptr, &lastcmdptr, FALSE))
     if (filter_test != NULL)
       {
       printf("Filter processing ended:\n");
-      if (*delivered)
+      if (action != NULL && *action != SPECIAL_NONE)
+        {
+        printf("  Filtering ended with the \"%s\" command.\n",
+          (*action == SPECIAL_FREEZE)? "freeze" : "fail");
+        }
+      else if (*delivered)
         printf("  Filtering set up at least one significant delivery "
                "or other action.\n"
                "  No other deliveries will occur.\n");
@@ -2546,7 +2554,7 @@ register int ch;
 BOOL yield, delivered;
 struct stat statbuf;
 address_item *generated = NULL;
-char *body, *error, *filebuf, *s, *tag;
+char *body, *error, *filebuf, *s;
 
 /* Read the filter file into store as will be done by the director
 in a real case. */
@@ -2569,51 +2577,55 @@ if (rc != statbuf.st_size)
 
 filebuf[statbuf.st_size] = 0;
 
-/* Check that the file starts with # Exim filter, as the director does. If
-it does not, treat it as an ordinary .forward file and check that. */
+/* Check that a user filter file starts with # Exim filter, as the director
+does. If it does not, treat it as an ordinary .forward file and check that.
+System filters do not do this check - they are always filters. */
 
-s = filebuf;
-tag = "# exim filter";
-while (isspace((uschar)*s)) s++;      /* Skips initial blank lines */
-for (; *tag != 0; s++, tag++)
+if (!is_system)
   {
-  if (*tag == ' ')
+  char *tag = "# exim filter";
+  s = filebuf;
+  while (isspace((uschar)*s)) s++;      /* Skips initial blank lines */
+  for (; *tag != 0; s++, tag++)
     {
-    while (*s == ' ' || *s == '\t') s++;
-    s--;
-    }
-  else if (tolower(*s) != tolower(*tag))
-    {
-    yield = parse_extract_addresses(filebuf,
-      deliver_domain,                 /* incoming domain for \name */
-      &generated,                     /* for generated addresses */
-      &error,                         /* for errors */
-      TRUE,                           /* no addresses => no error */
-      FALSE,                          /* don't recognize :blackhole: */
-      TRUE,                           /* do rewrite */
-      NULL,                           /* no check on includes */
-      NULL);                          /* fail on syntax errors */
-
-    if (yield != OK)
+    if (*tag == ' ')
       {
-      printf("exim: error in forward file: %s\n", error);
-      return FALSE;
+      while (*s == ' ' || *s == '\t') s++;
+      s--;
       }
-
-    if (generated == NULL)
-      printf("exim: no addresses generated from forward file\n");
-
-    else
+    else if (tolower(*s) != tolower(*tag))
       {
-      printf("exim: forward file generated:\n");
-      while (generated != NULL)
+      yield = parse_extract_addresses(filebuf,
+        deliver_domain,                 /* incoming domain for \name */
+        &generated,                     /* for generated addresses */
+        &error,                         /* for errors */
+        TRUE,                           /* no addresses => no error */
+        FALSE,                          /* don't recognize :blackhole: */
+        TRUE,                           /* do rewrite */
+        NULL,                           /* no check on includes */
+        NULL);                          /* fail on syntax errors */
+
+      if (yield != OK)
         {
-        printf("  %s\n", generated->orig);
-        generated = generated->next;
+        printf("exim: error in forward file: %s\n", error);
+        return FALSE;
         }
-      }
 
-    return TRUE;
+      if (generated == NULL)
+        printf("exim: no addresses generated from forward file\n");
+
+      else
+        {
+        printf("exim: forward file generated:\n");
+        while (generated != NULL)
+          {
+          printf("  %s\n", generated->orig);
+          generated = generated->next;
+          }
+        }
+
+      return TRUE;
+      }
     }
   }
 
@@ -2628,6 +2640,7 @@ message_body = body = store_malloc(message_body_visible + 1);
 message_body_end = store_malloc(message_body_visible + 1);
 s = message_body_end;
 body_len = 0;
+body_linecount = 0;
 
 if (!dot_ended && !feof(stdin))
   {
@@ -2635,10 +2648,10 @@ if (!dot_ended && !feof(stdin))
     {
     while ((ch = getc(stdin)) != EOF)
       {
+      if (ch == '\n') body_linecount++;
       if (body_len < message_body_visible) message_body[body_len++] = ch;
       *s++ = ch;
-      if (s > message_body_end + message_body_visible)
-        s = message_body_end;
+      if (s > message_body_end + message_body_visible) s = message_body_end;
       message_size++;
       }
     }
@@ -2650,7 +2663,7 @@ if (!dot_ended && !feof(stdin))
       switch (ch_state)
         {
         case 0:                         /* Normal state */
-        if (ch == '\n') ch_state = 1;
+        if (ch == '\n') { body_linecount++; ch_state = 1; }
         break;
 
         case 1:                         /* After "\n" */
@@ -2674,12 +2687,12 @@ if (!dot_ended && !feof(stdin))
         }
       if (body_len < message_body_visible) message_body[body_len++] = ch;
       *s++ = ch;
-      if (s > message_body_end + message_body_visible)
-        s = message_body_end;
+      if (s > message_body_end + message_body_visible) s = message_body_end;
       message_size++;
       }
     READ_END: ch = ch;  /* Some compilers don't like null statements */
     }
+  if (s == message_body_end || s[-1] != '\n') body_linecount++;
   }
 
 message_body[body_len] = 0;
@@ -2717,8 +2730,6 @@ while (*body != 0)
   body++;
   }
 
-
-
 /* Now pass the filter file to the function that interprets it. Because
 filter_test is not NULL, the interpreter will output comments about what
 it is doing, but an error message will have to be output here. No need to
