@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -20,7 +20,7 @@ extern void  accept_add_recipient(char *, char *, int, int);
 extern void  accept_bomb_out(char *);
 extern BOOL  accept_check_fs(int);
 extern BOOL  accept_msg(FILE *, BOOL);
-extern BOOL  accept_read_message_data_smtp(FILE *, FILE *, BOOL *);
+extern void  accept_swallow_smtp(FILE *);
 extern char *auth_b64encode(char *, int);
 extern int   auth_b64decode(char *, char **);
 extern BOOL  auth_call_pam(char *, char **);
@@ -29,8 +29,9 @@ extern char *auth_xtextencode(char *, int);
 extern int   auth_xtextdecode(char *, char **);
 
 extern int   child_close(pid_t, int);
-extern pid_t child_open(char **, char **, int, uid_t *, gid_t *, int *, int,
+extern pid_t child_open(char **, char **, int, uid_t *, gid_t *, int *, int *,
                char *, BOOL);
+extern pid_t child_open_exim(int *);
 
 extern void  daemon_go(void);
 extern void  debug_printf(char *, ...);
@@ -42,11 +43,13 @@ extern int   deliver_message(char *, BOOL, BOOL);
 extern void  deliver_restore_expansions(void);
 extern void  deliver_save_expansions(void);
 extern void  deliver_set_expansions(address_item *);
-extern void  deliver_setlocal(address_item *);
+extern BOOL  deliver_setlocal(address_item *);
 extern void  deliver_succeeded(address_item *);
-extern int   direct_check_fsc(director_instance *, address_item *);
 extern int   direct_address(address_item *, address_item **, address_item **,
                address_item **, address_item **, int);
+extern int   direct_check_fsc(director_instance *, address_item *);
+extern int   direct_check_prefix(char *, char *);
+extern int   direct_check_suffix(char *, char *);
 extern BOOL  direct_findgroup(char *, gid_t *);
 extern BOOL  direct_finduser(char *, struct passwd **, uid_t *);
 extern BOOL  direct_find_expanded_group(char *, char *, char *, gid_t *,
@@ -88,9 +91,9 @@ extern BOOL  header_checkname(header_line *, char *, int);
 
 extern char *host_and_ident(void);
 extern int   host_aton(char *, int *);
-extern void  host_build_hostlist(host_item **, char *);
+extern void  host_build_hostlist(host_item **, char *, BOOL);
 extern void  host_build_sender_fullhost(void);
-extern int   host_check_rbl(char *, char *, char **);
+extern int   host_check_rbl(char *, char *, char *, char **);
 extern BOOL  host_find_byname(host_item *, char **, BOOL);
 extern BOOL  host_find_bydns(host_item *, BOOL, BOOL, BOOL, BOOL, char **,
                BOOL *);
@@ -105,7 +108,7 @@ extern int   host_self_action(address_item *, host_item *, int, BOOL, char *);
 extern void  log_close_all(void);
 extern void  log_write(int, int, char *format, ...);
 
-extern BOOL  match_address_list(char *, int, char **, int, int, BOOL);
+extern BOOL  match_address_list(char *, int, char **, int, int *, BOOL);
 extern int   match_exists(char *);
 extern BOOL  match_isinlist(char *, char **, BOOL, BOOL, char **);
 extern BOOL  match_check_string(char *, char *, int, BOOL, BOOL, char **);
@@ -137,7 +140,6 @@ extern char *parse_fix_phrase(char *);
 
 extern BOOL  queue_action(char *, int, char **, int, int);
 extern void  queue_check_only(void);
-extern queue_filename *queue_get_spool_list(BOOL);
 extern void  queue_list(int, char **, int);
 extern void  queue_run(char *, char *, BOOL);
 
@@ -156,7 +158,7 @@ extern void  readconf_rewrites(void);
 extern BOOL  regex_match_and_setup(pcre *, char *, int, int);
 extern pcre *regex_must_compile(char *, BOOL, BOOL);
 extern void  retry_add_item(address_item *, char *, BOOL, char *, BOOL);
-extern BOOL  retry_check_address(host_item *, BOOL, char **, char **);
+extern BOOL  retry_check_address(char *, host_item *, BOOL, char **, char **);
 extern retry_config *retry_find_config(char *, char *, int, int);
 extern void  retry_update(address_item **, address_item **, address_item **);
 extern char *rewrite_address(char *, BOOL, BOOL);
@@ -168,7 +170,7 @@ extern int   route_address(address_item *, address_item **, address_item **,
                address_item **, int);
 extern void  route_init(void);
 extern BOOL  route_queue(address_item *, address_item **, address_item **,
-               router_instance *, char *);
+               router_instance *, ugid_block *, char *);
 extern BOOL  route_get_transport(transport_instance *, char *, address_item *,
                transport_instance **, char *, char *);
 extern BOOL  route_skip_driver(router_instance *, char *, char *, int, BOOL *,
@@ -221,7 +223,7 @@ extern char *string_format_size(int, char *);
 extern int   string_interpret_escape(char **);
 extern int   string_is_ip_address(char *, int *);
 extern char *string_log_address(address_item *, address_item *, BOOL, BOOL);
-extern char *string_nextinlist(char **, int, char *, int);
+extern char *string_nextinlist(char **, int *, char *, int);
 extern char *string_open_failed(int, char *, ...);
 extern char *string_printing(char *);
 extern char *string_sprintf(char *, ...);
