@@ -60,7 +60,7 @@ char *id_strtok __P3(char *, cp,
     /*
     ** Skip leading spaces
     */
-    while (isspace(*bp))
+    while (isspace((unsigned char)*bp))
 	bp++;
 
     /*
@@ -77,7 +77,7 @@ char *id_strtok __P3(char *, cp,
     ** Remove trailing spaces
     */
     *dc = *bp;
-    for (dc = bp-1; dc > cp && isspace(*dc); dc--)
+    for (dc = bp-1; dc > cp && isspace((unsigned char)*dc); dc--)
 	;
     *++dc = '\0';
 
