@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Thanks to Paul Kelly for contributing the original code for these
@@ -325,13 +325,14 @@ int
 mysql_find(void *handle, char *filename, char *query, int length,
   char **result, char **errmsg)
 {
+int sep = 0;
 char *server;
 char *list = mysql_servers;
 char buffer[512];
 
 DEBUG(9) debug_printf("MYSQL query: %s\n", query);
 
-while ((server = string_nextinlist(&list, ':', buffer, sizeof(buffer))) != NULL)
+while ((server = string_nextinlist(&list, &sep, buffer, sizeof(buffer))) != NULL)
   {
   BOOL defer_break;
   int rc = perform_mysql_search(query, server, result, errmsg, &defer_break);
@@ -351,8 +352,13 @@ return DEFER;
 *************************************************/
 
 /* The only characters that need to be quoted (with backslash) are newline,
-tab, carriage return, backspace, backslash itself, the quote characters,
-percent, and underscore.
+tab, carriage return, backspace, backslash itself, and the quote characters.
+Percent, and underscore and not escaped. They are only special in contexts
+where they can be wild cards, and this isn't usually the case for data inserted
+from messages, since that isn't likely to be treated as a pattern of any kind.
+Sadly, MySQL doesn't seem to behave like other programs. If you use something
+like "where id="ab\%cd" it does not treat the string as "ab%cd". So you really
+can't quote "on spec".
 
 Argument: the string to be quoted
 Returns:  the processed string
@@ -367,14 +373,14 @@ char *t = s;
 char *quoted;
 
 while ((c = *t++) != 0)
-  if (strchr("\n\t\r\b\'\"\\%_", c) != NULL) count++;
+  if (strchr("\n\t\r\b\'\"\\", c) != NULL) count++;
 
 if (count == 0) return s;
 t = quoted = store_get((int)strlen(s) + count + 1);
 
 while ((c = *s++) != 0)
   {
-  if (strchr("\n\t\r\b\'\"\\%_", c) != NULL)
+  if (strchr("\n\t\r\b\'\"\\", c) != NULL)
     {
     *t++ = '\\';
     switch(c)
