@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions for matching strings */
@@ -78,8 +78,9 @@ if (pattern[0] != '^')
 
   if (pattern[0] == '*')
     {
-    int patlen = (int)strlen(++pattern);
-    int slen = (int)strlen(s);
+    int patlen, slen;
+    patlen = (int)strlen(++pattern);
+    slen = (int)strlen(s);
     if (patlen > slen) return FALSE;
     yield = caseless?
       (strncmpic(s + slen - patlen, pattern, patlen) == 0) :
@@ -117,10 +118,10 @@ if (pattern[0] != '^')
     {
     char *orig_pattern = pattern;
     pattern += 7;
-    if (isdigit(*pattern))
+    if (isdigit((uschar)*pattern))
       {
       partial = 0;
-      while (isdigit(*pattern))
+      while (isdigit((uschar)*pattern))
         partial = partial * 10 + *pattern++ - '0';
       }
     else partial = 2;
@@ -173,7 +174,7 @@ if (pattern[0] != '^')
   if (pattern != NULL)                /* NULL => query type */
     {
     pattern = semicolon + 1;
-    while (isspace(*pattern)) pattern++;
+    while (isspace((uschar)*pattern)) pattern++;
     }
   handle = search_open(pattern, search_type, 0, NULL, NULL, &error);
   if (handle == NULL) log_write(0, LOG_PANIC_DIE, "%s", error);
@@ -226,6 +227,7 @@ Returns:         TRUE  if matched a non-negated item
                  TRUE  if hit end of list after a negated item
                  FALSE if matched a negated item
                  FALSE if hit end of list after a non-negated item
+                 FALSE, with search_find_defer set, if a lookup deferred
 */
 
 BOOL
@@ -233,6 +235,7 @@ match_isinlist(char *s, char **listptr, BOOL at_is_primary, BOOL caseless,
   char **valueptr)
 {
 BOOL yield = TRUE;
+int sep = 0;
 char *list = *listptr;
 char *sss;
 char *ot = NULL;
@@ -252,7 +255,7 @@ HDEBUG(9)
   }
 #endif
 
-while ((sss = string_nextinlist(&list, ':', buffer, sizeof(buffer))) != NULL)
+while ((sss = string_nextinlist(&list, &sep, buffer, sizeof(buffer))) != NULL)
   {
   char *ss = sss;
 
@@ -262,7 +265,7 @@ while ((sss = string_nextinlist(&list, ':', buffer, sizeof(buffer))) != NULL)
   if (*ss == '!')
     {
     yield = FALSE;
-    while (isspace(*(++ss)));
+    while (isspace((uschar)(*(++ss))));
     }
   else yield = TRUE;
 
@@ -306,11 +309,11 @@ while ((sss = string_nextinlist(&list, ':', buffer, sizeof(buffer))) != NULL)
       if ((ss = strchr(buffer, '#')) != NULL) *ss = 0;  /* trailing comment */
 
       ss = buffer + (int)strlen(buffer);                /* trailing space */
-      while (ss > buffer && isspace(ss[-1])) ss--;
+      while (ss > buffer && isspace((uschar)ss[-1])) ss--;
       *ss = 0;
 
       ss = buffer;
-      while (isspace(*ss)) ss++;                        /* leading space */
+      while (isspace((uschar)*ss)) ss++;                /* leading space */
 
       if (*ss == 0) continue;                           /* ignore empty */
 
@@ -318,7 +321,7 @@ while ((sss = string_nextinlist(&list, ':', buffer, sizeof(buffer))) != NULL)
       if (*ss == '!')                                   /* negation */
         {
         file_yield = !file_yield;
-        while (isspace(*(++ss)));
+        while (isspace((uschar)(*(++ss))));
         }
 
       if (*ss == '@' && ss[1] == 0 && at_is_primary) ss = primary_hostname_lc;
@@ -327,7 +330,6 @@ while ((sss = string_nextinlist(&list, ':', buffer, sizeof(buffer))) != NULL)
         fclose(f);
         return file_yield;
         }
-
       if (search_find_defer) break;
       }
 
@@ -338,12 +340,12 @@ while ((sss = string_nextinlist(&list, ':', buffer, sizeof(buffer))) != NULL)
     fclose(f);
     }
 
-  /* Deferral of any lookup is a disaster. */
+  /* Deferral of any lookup yields FALSE with search_find_defer set. */
 
   if (search_find_defer)
     {
-    log_write(0, LOG_MAIN|LOG_PANIC_DIE, "lookup of \"%s\" in \"%s\" failed "
-      "to complete", s, ss);
+    HDEBUG(9) debug_printf("%s lookup deferred\n", ot);
+    return FALSE;
     }
 
   } /* Loop for the next item on the top-level list */
@@ -392,6 +394,7 @@ char *ss, *file, *listptr;
 char buffer[1024];
 int yield = OK;
 int save_errno = 0;
+int sep = 0;
 uid_t saved_euid = (uid_t)0;
 gid_t saved_egid = (uid_t)0;
 BOOL restore_uid = FALSE;
@@ -410,7 +413,7 @@ DEBUG(9) debug_printf("require_files = %s\n", ss);
 
 listptr = ss;
 
-while ((file = string_nextinlist(&listptr, ':', buffer, sizeof(buffer)))
+while ((file = string_nextinlist(&listptr, &sep, buffer, sizeof(buffer)))
         != NULL)
   {
   int rc;
@@ -420,18 +423,18 @@ while ((file = string_nextinlist(&listptr, ':', buffer, sizeof(buffer)))
   if (*file == '!')
     {
     invert = TRUE;
-    while (isspace(*(++file)));
+    while (isspace((uschar)(*(++file))));
     }
 
   if (*file == '+')
     {
     eperm_is_absence = TRUE;
-    while (isspace(*(++file)));
+    while (isspace((uschar)(*(++file))));
 
     if (*file == '!')
       {
       invert = TRUE;
-      while (isspace(*(++file)));
+      while (isspace((uschar)(*(++file))));
       }
     }
 
@@ -584,7 +587,7 @@ used. */
 if (*ss == '!')
   {
   matchyield = am_no;
-  while (isspace(*(++ss)));
+  while (isspace((uschar)(*(++ss))));
   }
 else matchyield = am_yes;
 
@@ -643,6 +646,8 @@ else
 
     while (key != NULL && watchdog-- > 0)
       {
+      int sep = 0;
+
       if (!match_check_string(key, sdomain, -1, TRUE, FALSE, &result))
         goto NO_MATCH;
 
@@ -655,11 +660,11 @@ else
 
       ss = strrchr(list, ':');
       if (ss == NULL) ss = list; else ss++;
-      while (isspace(*ss)) ss++;
+      while (isspace((uschar)*ss)) ss++;
       if (*ss == '>')
         {
         *ss++ = 0;
-        while (isspace(*ss)) ss++;
+        while (isspace((uschar)*ss)) ss++;
         key = string_copy(ss);
         }
       else key = NULL;
@@ -667,7 +672,7 @@ else
       /* Look up the local parts provided by the list; negation is permitted.
       If a local part has to begin with !, a regex can be used. */
 
-      while ((ss = string_nextinlist(&list, ':', buffer, sizeof(buffer)))
+      while ((ss = string_nextinlist(&list, &sep, buffer, sizeof(buffer)))
              != NULL)
         {
         BOOL localyield = matchyield;
@@ -675,7 +680,7 @@ else
         if (*ss == '!')
           {
           localyield = (localyield == am_yes)? am_no : am_yes;
-          while (isspace(*(++ss)));
+          while (isspace((uschar)(*(++ss))));
           }
 
         if (match_check_string(address, ss, -1, TRUE, caseless, NULL))
@@ -793,7 +798,9 @@ Arguments:
   list          string list to check against
   expand_setup  controls setting up of $n variables - passed through
                 to match_check_string (q.v.)
-  separator     separator character for the list; may be 0 for one item
+  separator     pointer to separator character for the list;
+                may be 0 to get separator from the list;
+                may be -1 for one-item list
   ldr           what to return if a lookup defers
 
 Returns:        TRUE for a positive match, or end list after a negation;
@@ -803,7 +810,7 @@ Returns:        TRUE for a positive match, or end list after a negation;
 
 BOOL
 match_address_list(char *origaddress, int domain_offset, char **listptr,
-  int expand_setup, int separator, BOOL ldr)
+  int expand_setup, int *separator, BOOL ldr)
 {
 int llen;
 BOOL caseless = TRUE;
@@ -921,7 +928,7 @@ while ((ss = string_nextinlist(&list, separator, buffer, sizeof(buffer)))
     if (*ss == '!')
       {
       yes = FALSE;
-      while (isspace(*(++ss)));
+      while (isspace((uschar)(*(++ss))));
       }
     else yes = TRUE;
 
@@ -945,15 +952,15 @@ while ((ss = string_nextinlist(&list, separator, buffer, sizeof(buffer)))
       whitespace or eol before treating one as comment */
 
       if ((ss = strchr(fbuffer, '#')) != NULL &&
-          (ss[1] == 0 || isspace(ss[1])))
+          (ss[1] == 0 || isspace((uschar)ss[1])))
         *ss = 0;
 
       ss = fbuffer + (int)strlen(fbuffer);              /* trailing space */
-      while (ss > fbuffer && isspace(ss[-1])) ss--;
+      while (ss > fbuffer && isspace((uschar)ss[-1])) ss--;
       *ss = 0;
 
       ss = fbuffer;                                     /* leading space */
-      while (isspace(*ss)) ss++;
+      while (isspace((uschar)*ss)) ss++;
 
       if (*ss == 0) continue;                           /* ignore empty */
 
@@ -982,9 +989,11 @@ while ((ss = string_nextinlist(&list, separator, buffer, sizeof(buffer)))
         case am_ld:        /* lookup deferred */
         HDEBUG(2) debug_printf("%s %s (lookup deferred: %s)\n", ot,
           ldr? "yes" : "no", ss);
+        fclose(f);
         return ldr;
         }
       }
+    fclose(f);
     }
   }
 
@@ -1014,6 +1023,7 @@ int
 match_sender(char *s)
 {
 BOOL yield;
+int sep = 0;
 char *address = (sender_address == NULL)? "" : sender_address;
 
 if (s == NULL) return FAIL;
@@ -1023,7 +1033,7 @@ yield = match_address_list(
   (address[0] == 0)? 0 : strchr(address, '@') - address + 1,
   &s,
   FALSE,
-  ':',
+  &sep,
   FALSE);
 
 return yield? OK : search_find_defer? DEFER : FAIL;
@@ -1095,6 +1105,7 @@ char listbuffer[256];
 char subjbuffer[256];
 debug_level = 1;
 debug_file = stderr;
+debug_fd = fileno(debug_file);
 big_buffer = malloc(big_buffer_size);
 
 primary_hostname_lc = "local.host";
@@ -1171,6 +1182,7 @@ printf("Testing match_address_list()\n");
 for (;;)
   {
   int len;
+  int sep = 0;
   char *list = listbuffer;
 
   printf("List: ");
@@ -1189,7 +1201,7 @@ for (;;)
     subjbuffer[len-1] = 0;
     s = strchr(subjbuffer, '@');
     domain_offset = (s == NULL)? 0 : s - subjbuffer + 1;
-    if (match_address_list(subjbuffer, domain_offset, &list, -1, ':', FALSE))
+    if (match_address_list(subjbuffer, domain_offset, &list, -1, &sep, FALSE))
       printf("OK\n");
     else
       printf("Failed%s\n", search_find_defer? " (lookup deferred)" : "");
