@@ -196,20 +196,21 @@ for (rule = rewrite_rules;
     break;
     }
 
-  /* Validate what has been generated, qualifying if permitted to do so. */
+  /* Validate what has been generated, qualifying if permitted to do so.
+  Only envelope from is permitted to be rewritten as <>.*/
 
   if ((rule->flags & rewrite_qualify) != 0)
     new = rewrite_address_qualify(new, TRUE);
 
   newparsed = parse_extract_address(new, &error, &start, &end, &pdomain,
-    FALSE);
+    flag == rewrite_envfrom);
 
   if (newparsed == NULL)
     {
     log_write(0, LOG_MAIN|LOG_PANIC, "Rewrite of %s yielded unparseable "
       "address: %s in address %s", subject, error, new);
     }
-  else if (pdomain == 0)
+  else if (pdomain == 0 && (flag != rewrite_envfrom || *newparsed != 0))
     {
     log_write(0, LOG_MAIN|LOG_PANIC, "Rewrite of %s yielded unqualified "
       "address \"%s\"", subject, new);
