@@ -160,6 +160,8 @@ if (expanded == NULL)
   return ERROR;
   }
 
+DEBUG(2) debug_printf("command is: %s\n", expanded);
+
 /* Split on white space to make the arguments. */
 
 for (i = 0; *s != 0 && i < 19; i++)
@@ -270,7 +272,8 @@ if (strcmpic(returned[0], "OK") != 0)
   addr->message = string_sprintf("%s router: %s %s %s %s", rblock->name,
     returned[1], returned[2], returned[3], returned[4]);
 
-  if (strcmpic(returned[0], "fail") == 0) return FAIL;
+  if (strcmpic(returned[0], "decline") == 0) return DECLINE;
+  if (strcmpic(returned[0], "fail") == 0) return DECLINE;     /* Obsolete */
   if (strcmpic(returned[0], "forcefail") == 0) return FORCEFAIL;
   if (strcmpic(returned[0], "defer") == 0) return DEFER;
 
@@ -310,7 +313,7 @@ if (returned[1][0] != 0 && strcmp(returned[1], "+") != 0)
 /* Second field was empty or "+"; get the transport for this router, if
 any. It might be expanded, or it might be unset (which is not an error). */
 
-if (!route_get_transport(rblock->transport, rblock->expand_transport,
+else if (!route_get_transport(rblock->transport, rblock->expand_transport,
       addr, &transport, rblock->name, NULL))
   return ERROR;
 
@@ -367,7 +370,7 @@ if (returned[2][0] != 0 && strcmp(returned[2], "+") != 0)
         {
         DEBUG(2) debug_printf("%s router timed out & pass_on_timeout set\n",
           rblock->name);
-        return FAIL;
+        return PASS;
         }
       return DEFER;
       }
