@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1999 */
+/* Copyright (c) University of Cambridge 1995 - 2000 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* A set of functions to search databases in various formats. An open
@@ -166,12 +166,12 @@ return 0;
 
 /* When search_open is called it caches the "file" that it opens in
 search_tree. The name of the tree node is a concatenation of the search type
-with the file name. For query-style lookups, the file name is empty. Real
-files are normally closed only when this tidyup routine is called, typically at
-the end of sections of code where a number of lookups might occur. However, if
-too many files are open simultaneously, some get closed beforehand. They can't
-be removed from the tree. There is also a general tidyup function which is
-called for the lookup driver, if it exists.
+with the file name. For query-style lookups, the file name is empty. Real files
+are normally closed only when this tidyup routine is called, typically at the
+end of sections of code where a number of lookups might occur. However, if too
+many files are open simultaneously, some get closed beforehand. They can't be
+removed from the tree. There is also a general tidyup function which is called
+for the lookup driver, if it exists.
 
 First, there is an internal, recursive subroutine.
 
@@ -707,7 +707,8 @@ if (yield == NULL && do_at)
   char *atat = strrchr(keystring, '@');
   if (atat != NULL && atat > keystring)
     {
-    int savechar = *(--atat);
+    int savechar;
+    savechar = *(--atat);
     *atat = '*';
 
     DEBUG(9) debug_printf("trying default match %s\n", atat);
