@@ -621,11 +621,30 @@ if ((options & topt_no_headers) == 0)
     }
 
   /* Add on any address-specific headers. If there are multiple addresses,
-  they will all have the same headers in order to be batched. */
+  they will all have the same headers in order to be batched. The headers
+  are chained in reverse order of adding (so several addresses from the
+  same alias might share some of them) but we want to output them in the
+  opposite order. This is a bit tedious, but there shouldn't be very many
+  of them. We just walk the list twice, reversing the pointers each time,
+  but on the second time, write out the items. */
 
   if (addr != NULL)
-    for (h = addr->extra_headers; h!= NULL; h = h->next)
-      if (!write_chunk(fd, h->text, h->slen, use_crlf)) return FALSE;
+    {
+    int i;
+    header_line *hprev = addr->extra_headers;
+    header_line *hnext;
+    for (i = 0; i < 2; i++)
+      {
+      for (h = hprev, hprev = NULL; h != NULL; h = hnext)
+        {
+        hnext = h->next;
+        h->next = hprev;
+        hprev = h;
+        if (i == 1 && !write_chunk(fd, h->text, h->slen, use_crlf))
+          return FALSE;
+        }
+      }
+    }
 
   /* If a string containing additional headers exists, expand it and write
   out the result. This is done last so that if it (deliberately or accidentally)
