@@ -98,7 +98,7 @@ smtp_transport_options_block smtp_transport_option_defaults = {
   NULL,            /* authenticate_hosts */
   0,               /* batch_max */
   5*60,            /* command_timeout */
-  0,               /* connect_timeout; 0 => system default */
+  5*60,            /* connect_timeout; shorter system default overrides */
   5*60,            /* data timeout */
   10*60,           /* final timeout */
   1024,            /* size_addition */
@@ -1621,9 +1621,10 @@ for (cutoff_retry = 0; expired &&
     we don't actually want to attempt any deliveries. When doing a queue run,
     queue_smtp_domains is unset. If there is a lookup defer in queue_smtp_
     domains, proceed as if the domain were not in it. We don't want to hold up
-    all SMTP deliveries! */
+    all SMTP deliveries! Except when doing a two-stage queue run, don't do
+    this if forcing. */
 
-    if (!deliver_force && (queue_smtp ||
+    if ((!deliver_force || queue_2stage) && (queue_smtp ||
         match_isinlist(addrlist->domain, &queue_smtp_domains,TRUE,TRUE,NULL)))
       {
       expired = FALSE;
